//
//  DetailViewController.m
//  ToDo-RestKit
//
//  Created by Rodger Higgins on 4/7/13.
//  Copyright (c) 2013 Rodger Higgins. All rights reserved.
//

#import "TaskViewController.h"
#import "TaskDataModel.h"
#import "Task.h"
#import "DatePickerViewController.h"


@interface TaskViewController () {
    UIPopoverController *_popoverController;
    UIDatePicker *_datePicker;
    
    DatePickerViewController *_dpvc;
    
    TaskDataModel *_tdm;
    BOOL _startDate;
}


@property (strong, nonatomic) UIPopoverController *masterPopoverController;
- (void)configureView;
@end

@implementation TaskViewController

#pragma mark - Managing the detail item

- (void)setTaskItem:(Task *)newTaskItem
{
    if (_taskItem != newTaskItem) {
        _taskItem = newTaskItem;
    }
    
    // Update the view.
    [self configureView];

    if (self.masterPopoverController != nil) {
        [self.masterPopoverController dismissPopoverAnimated:YES];
    }        
}

- (void)configureView
{
    // Update the user interface for the detail item.

    if (self.taskItem) {
        self.titleTextEdit.text = self.taskItem.title;
        self.projectTextEdit.text = self.taskItem.project;
        self.contextTextEdit.text = self.taskItem.context;
        
        [self.startDateButton setTitle:[self.taskItem startDateAsString] forState:UIControlStateNormal];
        [self.dueDateButton setTitle:[self.taskItem dueDateAsString] forState:UIControlStateNormal];
        
        self.noteTextView.text = self.taskItem.note;
        [self setCompletedButtonSelected:self.taskItem.complete];
    }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [self configureView];
    
    [self.noteTextView.layer setBorderColor:[[UIColor grayColor] CGColor]];
    [self.noteTextView.layer setBorderWidth:2.3];
    [self.noteTextView.layer setCornerRadius:10];
    
    // Setup delegates
    self.titleTextEdit.delegate = self;
    self.projectTextEdit.delegate = self;
    self.contextTextEdit.delegate = self;
    self.noteTextView.delegate = self;
    
    _tdm = [TaskDataModel sharedDataModel];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = NSLocalizedString(@"Detail", @"Task");
    }
    return self;
}
							
#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Tasks", @"Tasks");
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

- (BOOL)splitViewController: (UISplitViewController*)svc
   shouldHideViewController:(UIViewController *)vc
              inOrientation:(UIInterfaceOrientation)orientation
{
    return NO;
}


#pragma mark - Text Editing Methods

- (void)textFieldDidEndEditing:(UITextField *)textField
{
    if ( textField == self.titleTextEdit ) {
        self.taskItem.title = textField.text;
    } else if ( textField == self.projectTextEdit ) {
        self.taskItem.project = textField.text;
    } else if ( textField == self.contextTextEdit ) {
        self.taskItem.context = textField.text;
    }
    
    [_tdm saveMainContext];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return YES;
}

- (void)textViewDidBeginEditing:(UITextView *)textView
{
    CGRect r = CGRectMake(0, -textView.frame.origin.y+10, self.view.frame.size.width, self.view.frame.size.height);
    
    [self.navigationItem setRightBarButtonItem:[[UIBarButtonItem alloc] initWithTitle:@"Done" style:UIBarButtonItemStylePlain target:self action:@selector(dismissKeyboardTextView)] animated:YES];
    
    [UIView animateWithDuration:0.50 animations:^{
        [self.view setFrame:r];
    }];
}

- (void)textViewDidEndEditing:(UITextView *)textView
{
    // Only a note text view
    self.taskItem.note = textView.text;
    [_tdm saveMainContext];

    [UIView animateWithDuration:0.50 animations:^{
        [self.view setFrame:CGRectMake(0,0,self.view.frame.size.width,self.view.frame.size.height)];
    }];
    [self.navigationItem setRightBarButtonItem:nil animated:YES];
}

- (void)dismissKeyboardTextView
{
    [self.noteTextView resignFirstResponder];
}

- (IBAction)completeButtonTouched:(id)sender
{
    [self setCompletedButtonSelected:!self.completedButton.selected];
    
    // Update the taskItem
    self.taskItem.complete = self.completedButton.selected;
    [_tdm saveMainContext];
}

- (void)setCompletedButtonSelected:(BOOL)selected
{
    self.completedButton.selected = selected;
    
    if ( self.completedButton.selected ) {
        // Load selected image
        [self.completedButton setImage:[UIImage imageNamed:@"checked.png"] forState:UIControlStateNormal];
    } else {
        // Load default image
        [self.completedButton setImage:[UIImage imageNamed:@"unchecked.png"] forState:UIControlStateNormal];
    }
}


#pragma mark -
#pragma mark OCCalendar Delegate Methods

- (void)completedWithStartDate:(NSDate *)startDate endDate:(NSDate *)endDate
{
    DLog(@"startDate: %@, endDate: %@", startDate, endDate);
}

- (IBAction)selectStartDate:(id)sender {
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        [self loadiPhoneDatePicker:YES];
    } else {
        [self loadDatePopover:YES];
    }
}

- (IBAction)selectEndDate:(id)sender
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        [self loadiPhoneDatePicker:NO];
    } else {
        [self loadDatePopover:NO];
    }
}



#pragma mark - iPad date picker popover methods

- (void)loadDatePopover:(BOOL)start
{
    _startDate = start;
    
    UIViewController* popoverContent = [[UIViewController alloc] init]; //ViewController
    
    _datePicker=[[UIDatePicker alloc]init];//Date picker
    _datePicker.frame=CGRectMake(0, 0, 320, 200);
    _datePicker.datePickerMode = UIDatePickerModeDate;
    _datePicker.date = self.taskItem.due;
    [_datePicker addTarget:self action: @selector(datePicked) forControlEvents:UIControlEventValueChanged];
    
    popoverContent.view = _datePicker;
    _popoverController = [[UIPopoverController alloc] initWithContentViewController:popoverContent];
    _popoverController.delegate=self;
    
    [_popoverController setPopoverContentSize:CGSizeMake(320, 156) animated:YES];
    
    if ( _startDate ) {
        [_popoverController presentPopoverFromRect:self.startDateButton.frame inView:self.view permittedArrowDirections:UIPopoverArrowDirectionUp animated:YES];
    } else {
        [_popoverController presentPopoverFromRect:self.dueDateButton.frame inView:self.view permittedArrowDirections:UIPopoverArrowDirectionUp animated:YES];
    }
}

- (void)popoverControllerDidDismissPopover:(UIPopoverController *)popoverController
{
    DLog(@"DidDismissPopover: %@", self.taskItem.start);
    
    _datePicker = nil;
    _popoverController = nil;
}

- (void)datePicked
{
    NSDate *d = _datePicker.date;
    
    if ( _startDate ) {
        self.taskItem.start = d;
        [self.startDateButton setTitle:[self.taskItem startDateAsString] forState:UIControlStateNormal];
    } else {
        self.taskItem.due = d;
        [self.dueDateButton setTitle:[self.taskItem dueDateAsString] forState:UIControlStateNormal];
    }
    
    [_tdm saveMainContext];
}



#pragma mark - iPhone Date Picker Methods

- (void) iPhoneDatePicked
{
    DLog(@"Date: %@", _dpvc.date);

    if ( _startDate ) {
        self.taskItem.start = _dpvc.date;
    } else {
        self.taskItem.due = _dpvc.date;
    }
    
    [self.navigationController popViewControllerAnimated:YES];
    [_tdm saveMainContext];
    [self configureView];
    
    _dpvc = nil;
}

- (void)loadiPhoneDatePicker:(BOOL)start
{
    _startDate = start;
    
    _dpvc = [[DatePickerViewController alloc] initWithNibName:@"DatePickerViewController" bundle:nil];
    
    if ( _startDate ) {
        [_dpvc setDate:@"Start Date" date:self.taskItem.start];
        [_dpvc setTitle:@"Start Date"];
    } else {
        [_dpvc setDate:@"Due Date" date:self.taskItem.due];
        [_dpvc setTitle:@"Due Date"];
    }
    
    UIBarButtonItem *b = [[UIBarButtonItem alloc] initWithTitle:@"Save" style:UIBarButtonItemStyleDone target:self action:@selector(iPhoneDatePicked)];
    
    _dpvc.navigationItem.rightBarButtonItem = b;
    
    [self.navigationController pushViewController:_dpvc animated:YES];
}

@end
