//
//  MasterViewController.m
//  ToDo-RestKit
//
//  Created by Rodger Higgins on 4/7/13.
//  Copyright (c) 2013 Rodger Higgins. All rights reserved.
//

#import "TasksListViewController.h"
#import "ToDoCell.h"
#import "TaskViewController.h"
#import "TaskDataModel.h"
#import "Task.h"

@interface TasksListViewController () {
    TaskDataModel *_tdm;
}
@end

@implementation TasksListViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = NSLocalizedString(@"Tasks", @"Tasks");
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
            self.clearsSelectionOnViewWillAppear = NO;
            self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
        }
        
        _dataModelUp = NO;
        
        _tdm = [TaskDataModel sharedDataModel];
        
        [_tdm registerDataModelOnline:self selector:@selector(receiveDataModelOnline:) object:nil];
    }
    return self;
}
							
- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    self.navigationItem.leftBarButtonItem = self.editButtonItem;

    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(insertNewObject:)];
    self.navigationItem.rightBarButtonItem = addButton;
    
    // Register ToDoCell
    [self.tableView registerClass:[ToDoCell class] forCellReuseIdentifier:@"ToDoCell"];
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)insertNewObject:(id)sender
{
    Task *t = [_tdm newTask];
    t.title = @"New Task Title";
    t.start = [NSDate date];
    t.due = [NSDate date];
    t.project = @"My Project";
    t.context = @"At My Desk";
    t.note = @"New Note";
    [_tdm saveMainContext];
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [[self.fetchedResultsController sections] count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [[self.fetchedResultsController sections][section] numberOfObjects];
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"ToDoCell";
    ToDoCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    Task *t = [self.fetchedResultsController objectAtIndexPath:indexPath];
    [cell setTask:t];
    cell.delegate = self;
    
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 67;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        Task *t = [self.fetchedResultsController objectAtIndexPath:indexPath];
        [_tdm deleteTask:t];
        [_tdm saveMainContext];
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }
}

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [_tdm saveMainContext];
    Task *t = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
	    if (!self.detailViewController) {
	        self.detailViewController = [[TaskViewController alloc] initWithNibName:@"DetailViewController_iPhone" bundle:nil];
	    }
	    self.detailViewController.taskItem = t;
        [self.navigationController pushViewController:self.detailViewController animated:YES];
    } else {
        self.detailViewController.taskItem = t;
    }
}




#pragma mark -
#pragma mark CoreData stuff

- (NSFetchedResultsController *)fetchedResultsController
{
    // Initialize _fetchedResultsController if not previously accessed
    if ( _fetchedResultsController == nil ) {
        if ( _dataModelUp ) {
            NSManagedObjectContext *moc = [_tdm mainContext];
            
            NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:@"Task"];
            NSSortDescriptor *sort = [[NSSortDescriptor alloc] initWithKey:@"title" ascending:YES];
            [fetchRequest setSortDescriptors:[NSArray arrayWithObject:sort]];
            
            NSFetchedResultsController *frc = [[NSFetchedResultsController alloc]
                                               initWithFetchRequest:fetchRequest
                                               managedObjectContext:moc
                                               sectionNameKeyPath:nil
                                               cacheName:nil];
            
            NSError *error = nil;
            ZAssert([frc performFetch:&error],
                    @"Unresolved error %@\n%@", [error localizedDescription],
                    [error userInfo]);
            
            _fetchedResultsController = frc;
            _fetchedResultsController.delegate = self;
        }
    }
    
    return _fetchedResultsController;
}

- (void)receiveDataModelOnline:(NSNotification *)notification
{
    _dataModelUp = YES;
    DLog(@"Notification received");
    [self.tableView reloadData];
}

- (void)selectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSLog(@"IndexPath: %@", indexPath);
    [self.tableView selectRowAtIndexPath:indexPath animated:YES scrollPosition:UITableViewScrollPositionNone];
    if ([self.tableView.delegate respondsToSelector: @selector (tableView:didSelectRowAtIndexPath:)])
        [self.tableView.delegate tableView:self.tableView didSelectRowAtIndexPath:indexPath];
    
}


#pragma mark -
#pragma mark NSFetchedResultsDelegate methods

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView beginUpdates];
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView endUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller
   didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath
     forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    switch (type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            [self selectRowAtIndexPath:newIndexPath];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            {
                ToDoCell *cell = (ToDoCell *)[self.tableView cellForRowAtIndexPath:indexPath];
                Task *t = [self.fetchedResultsController objectAtIndexPath:indexPath];
                [cell setTask:t];
            }
            break;
            
        case NSFetchedResultsChangeMove:
            [self.tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [self.tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        default:
            break;
    }
}


#pragma mark - ToDoCellDelegate method

- (void)completedState:(id)sender state:(BOOL)state
{
    NSIndexPath *ip = [self.tableView indexPathForCell:sender];
    
    Task *t = [self.fetchedResultsController objectAtIndexPath:ip];
    t.complete = state;
    [_tdm saveMainContext];
    
    [(ToDoCell *)sender setTask:t];
}
@end
