//
//  TaskDataModel.m
//  ToDo-RestKit
//
//  Created by Rodger Higgins on 4/9/13.
//  Copyright (c) 2013 Rodger Higgins. All rights reserved.
//

#import "TaskDataModel.h"

@implementation TaskDataModel


+ (TaskDataModel *)sharedDataModel {
    static TaskDataModel *_sharedTaskDataModel = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedTaskDataModel = [[TaskDataModel alloc] init];
    });
    
    return _sharedTaskDataModel;
}
- (NSString *)documentDirectory
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    return [paths objectAtIndex:0];
}

- (NSString *)modelName
{
    return @"Task";
}

- (NSString *)pathToModel
{
    return [[NSBundle mainBundle] pathForResource:[self modelName] ofType:@"momd"];
}

- (NSString *)storeFilename
{
    return [[self modelName] stringByAppendingPathExtension:@"sqlite"];
}

- (NSString *)pathToLocalStore
{
    return [[self documentDirectory] stringByAppendingPathComponent:[self storeFilename]];
}

- (NSDictionary *)persistentStoreOptions
{
    return [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool:YES], NSMigratePersistentStoresAutomaticallyOption,
            [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil];
}

- (NSString *)storeType
{
    return NSSQLiteStoreType;
}


#pragma mark - Main Context Methods

- (NSError *)saveMainContext
{
    NSError *error;
    [self.mainContext save:&error];
    return error;
}

#pragma mark - Task Model Methods

- (id)newTask
{
    return [NSEntityDescription insertNewObjectForEntityForName:@"Task" inManagedObjectContext:self.mainContext];
}

- (void)deleteTask:(Task *)task
{
    [self.mainContext deleteObject:task];
}
@end
