//
//  ToDoCell.m
//  ToDo-RestKit
//
//  Created by Rodger Higgins on 4/15/13.
//  Copyright (c) 2013 Rodger Higgins. All rights reserved.
//

#import "ToDoCell.h"
#import "Task.h"

@interface ToDoCell () {
    Task *_task;
}
@end


@implementation ToDoCell
@synthesize mainTextLabel = _mainTextLabel;
@synthesize noteTextLabel = _noteTextLabel;
@synthesize startDateLabel = _startDateLabel;
@synthesize dueDateLabel = _dueDateLabel;
@synthesize completedButton = _completedButton;
@synthesize height = _height;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        // Initialization code
        [self addMainTextLabel];
        [self addNoteTextLabel];
        [self addStartDateLabel];
        [self addDueDateLabel];
        [self addCompletedButton];
        
    }
    return self;
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated
{
    [super setSelected:selected animated:animated];

    // Configure the view for the selected state
}



#pragma mark - Layout Subviews

- (void)layoutSubviews
{
    [super layoutSubviews];
    
    const CGFloat HorizontalMargin = 10;
    const CGFloat VerticalMargin = 4;
    
    CGFloat x = HorizontalMargin;
    CGFloat y = VerticalMargin;
    
    // Completed Button on the left
    CGFloat height = self.contentView.frame.size.height - (2 * VerticalMargin);
    CGFloat width = self.contentView.frame.size.height - (2 * VerticalMargin);
    self.completedButton.frame = CGRectMake( x, y, width, height );
    
    // Main Title View
    x = width + 2 * HorizontalMargin;
    width = self.contentView.bounds.size.width - x - 2 * HorizontalMargin;
    height = [self.mainTextLabel.text sizeWithFont:self.mainTextLabel.font].height;
    self.mainTextLabel.frame = CGRectMake(x, y, width, height);
    y += VerticalMargin + height;
    
    // Start View
    width = width / 2.0 - HorizontalMargin;
    height = [self.startDateLabel.text sizeWithFont:self.startDateLabel.font].height;
    self.startDateLabel.frame = CGRectMake(x, y, width, height);
    
    // Due View
    self.dueDateLabel.frame = CGRectMake(x + width + HorizontalMargin, y, width, height);
    y += VerticalMargin + height;
    
    // Note View
    width = self.contentView.bounds.size.width - x - 2 * HorizontalMargin;
    height = [self.noteTextLabel.text sizeWithFont:self.noteTextLabel.font].height;
    self.noteTextLabel.frame = CGRectMake(x, y, width, height);
    
    self.height = y + height + VerticalMargin;
//    NSLog(@"Height: %f", self.height);
}



#pragma mark - Initialize Cell Methods

- (void)addMainTextLabel
{
    self.mainTextLabel = [[UILabel alloc] init];
    self.mainTextLabel.font = [UIFont boldSystemFontOfSize:16];
    self.mainTextLabel.textColor = [UIColor colorWithWhite:0.45 alpha:1.0];
    
    [self.contentView addSubview:self.mainTextLabel];
}

- (void)addNoteTextLabel
{
    self.noteTextLabel = [[UILabel alloc] init];
    self.noteTextLabel.font = [UIFont systemFontOfSize:10];
    self.noteTextLabel.textColor = [UIColor colorWithWhite:0.30 alpha:1.0];
    
    [self.contentView addSubview:self.noteTextLabel];
}

- (void)addStartDateLabel
{
    self.startDateLabel = [[UILabel alloc] init];
    self.startDateLabel.font = [UIFont systemFontOfSize:14];
    self.startDateLabel.textColor = [UIColor colorWithWhite:0.30 alpha:1.0];
    
    [self.contentView addSubview:self.startDateLabel];
}

- (void)addDueDateLabel
{
    self.dueDateLabel = [[UILabel alloc] init];
    self.dueDateLabel.font = [UIFont systemFontOfSize:14];
    self.dueDateLabel.textColor = [UIColor colorWithWhite:0.30 alpha:1.0];
    
    [self.contentView addSubview:self.dueDateLabel];
}

- (void)addCompletedButton
{
    self.completedButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.completedButton setImage:[UIImage imageNamed:@"unchecked.png"] forState:UIControlStateNormal];
    [self.contentView addSubview:self.completedButton];
    
    [self.completedButton addTarget:self action:@selector(completedButtonTouched:) forControlEvents:UIControlEventTouchUpInside];
}

- (IBAction)completedButtonTouched:(id)sender
{
    self.completedButton.selected = !self.completedButton.selected;
    
    [self setImageBySelectedState:self.completedButton.selected];
    if ([self.delegate respondsToSelector:@selector(completedState:state:)]) {
        [self.delegate completedState:self state:self.completedButton.selected];
    }
}

- (void)setImageBySelectedState:(BOOL)selected
{
    if (selected) {
        [self.completedButton setImage:[UIImage imageNamed:@"checked.png"] forState:UIControlStateNormal];
    } else {
        [self.completedButton setImage:[UIImage imageNamed:@"unchecked.png"] forState:UIControlStateNormal];
    }
}

- (void)setTask:(Task *)task
{
    if ( _task != task ) {
        _task = task;
    }
    
    self.mainTextLabel.text = _task.title;
    self.noteTextLabel.text = _task.note;
    self.startDateLabel.text = [_task startDateAsString];
    self.dueDateLabel.text = [_task dueDateAsString];
    self.completedButton.selected = _task.complete;
    [self setImageBySelectedState:_task.complete];
}
@end

